;-----------------------------------------------------------------------
;                  Program  Wiegand to RS232 Converter
;-----------------------------------------------------------------------
;    Program controls one card reader supporting Wiegand 26 or Wiegand 37
; interface. Transmitted by the card reader data (i.e. card number) are
; checked for errors and retransmitted via RS232 interface to PC. Both
; interfaces (i.e. Wiegand and RS232) are implemented in software as
; interrupt service routines. Wiegand interface uses external interrupt
; INT0 while RS232 interface uses timer T0 overflow interrupt.
;
; Technical data:
;    - Wiegand data format: Wiegand26 or Wiegand37
;    - RS232:               full duplex
;    - RS232 speed:         1200 bit/s
;    - RS232 data format:   8N1
;
; Target CPU: AT90S1200
; Fosc:       7.3728MHz
; Version:    1.00
;
; history:
;    2004.02.21 - start of the project
;    2004.03.19 - final version
;-----------------------------------------------------------------------

         .include "1200def.inc"        ; Register/Bit Definitions for the AT90S1200
 
; ---- Constants
.equ T0_period    = 24                 ; period of timer T0 interrupts
                                       ; (208.33us @ Fosc=7.3728MHz & T0psc=64)
.equ Wgnd_tout    = 96                 ; the next Wiegand bit reception time-out time
                                       ;      (Wgnd_tout*208.33us=20ms)
                                       
.equ char_STX     = 0x02
.equ char_ETX     = 0x03
                                   
; ---- Variables
.def SREGcopy     = r0                 ; copy of the status register (SREG)
.def Tx_char      = r1                 ; RS transmitter buffer
.def Rx_char      = r2                 ; RS receiver buffer
.def time_Rx      = r3                 ; moment of reading RS bit
.def Wiegand_B0   = r4                 ; \ 
.def Wiegand_B1   = r5                 ; |
.def Wiegand_B2   = r6                 ; | Wiegand buffer
.def Wiegand_B3   = r7                 ; |
.def Wiegand_B4   = r8                 ; /
.def reg_A        = r9                 ; \ 
.def reg_B        = r10                ; |
.def reg_C        = r11                ; | main program working registers
.def reg_D        = r12                ; |
.def reg_E        = r13                ; /
.def temp         = r16                ; main program temporary register
.def tempINT      = r17                ; interrupt service temporary register
.def time_count   = r18                ; timer interrupts counter
.def flags        = r19                ; flags register
.equ f_charOK   = 2                    ; character in Rx_char buffer
.equ f_Rx_busy  = 1                    ; RS character reception in prograss
.equ f_Tx_busy  = 0                    ; RS character transmition in progress
.def Tx_bit_count = r20                ; sent bits counter
.def Rx_bit_count = r21                ; received bits counter
.def Wgnd_count   = r22                ; read Wiegand bits counter
.def time_Wgnd    = r23                ; the next Wiegand bit reception time-out timer
.def Tx_state     = r24                ; RS transmitter state machine
.def Rx_state     = r25                ; RS receiver state machine
.def loop_count   = r26                ; main program working register

; ---- I/O Ports Definitions
.equ RS_RxD    = 1                     ; PB1 - RS232 Receive Data Line
.equ RS_TxD    = 0                     ; PB0 - RS232 Transmit Data Line

.equ W_Beeper  = 5                     ; PD5 - Wiegand Beeper Control Line
.equ W_LED_G   = 4                     ; PD4 - Wiegand Green LED Control Line
.equ W_LED_R   = 3                     ; PD3 - Wiegand Red LED Control Line
.equ W_Data_1  = 1                     ; PD1 - Wiegand Data_1 Line
.equ W_Data_0  = 0                     ; PD0 - Wiegand Data_0 Line


;-----------------------------------------------------------------------
;                               Program
;-----------------------------------------------------------------------
         .org 0x00
          rjmp    reset                ; Main Program
          rjmp    ext_INT0             ; External Interrupt 0 Service
;         rjmp    T0_ovl               ; Timer 0 Overflow Service
;         rjmp    AnComp               ; Analog Comparator Service


;-----------------------------------------------------------------------
;         Timer T0 Overflow Service Subroutine (vector 0x02)
;-----------------------------------------------------------------------
; Timer T0 generates interrupts four times during every RS232 bit
;                     _____                     ___
;   RS232:                 |___________________|
;
;   T0ovl interrupt:    ___|____|____|____|____|___
;   time_count:            0    1    2    3    0
;
; RS transmition starts always at time_count=0 while RS reception is done
; by polling and starts two timer interrupts later, after the one in which
; the falling edge of the START bit has been detected
;
T0_ovl:   in      SREGcopy,SREG        ; save SREG
          ldi     tempINT,(256-T0_period)
          out     TCNT0,tempINT        ; reload T0
          
          inc     time_count           ; count timer interrupts
          andi    time_count,0x03      ; time_count counts modulo 4
          
          cpi     time_Wgnd,0x00       ; Wiegand time-out timer on?
          breq    T0rs                 ; jump if not
          dec     time_Wgnd            ; count time-out
          brne    T0rs                 ; Wiegand time-out?
          ldi     Tx_state,0x01        ; start RS transmitter state machine
          
T0rs:     sbrc    flags,f_Tx_busy      ; transmit char?
          rcall   send_char
          
          bst     flags,f_Rx_busy      ; T=f_Rx_char
          brtc    T0pollRxD            ; receive char?
          rcall   get_char
          rjmp    T0end
          
T0pollRxD:sbic    PINB,RS_RxD          ; START bit (RxD=0)?
          rjmp    T0end                ; jump if not
          mov     tempINT,time_count   ; \ 
          subi    tempINT,(0x100-0x02) ; | calculate time of the middle of the bit
          andi    tempINT,0x03         ; |      time_Rx=(time_count+2)mod4
          mov     time_Rx,tempINT      ; /
          sbr     flags,(1<<f_Rx_busy) ; activate RS receiver
          
T0end:    out     SREG,SREGcopy        ; restore SREG
          reti


; ---- RS transmit character
; Transmition of the RS232 bit starts always at time_count=0
; Tx_char is sent in format 8N1 as 10 bits:
;              START b0 b1 b2 b3 b4 b5 b6 b7 STOP
; Cleared flag f_Tx_busy signals the end of the RS transmition
;
send_char:cpi     time_count,0x00
          breq    sendSTART            ; start of the bit?
          ret
          
sendSTART:cpi     Tx_bit_count,0x00    ; bit START?
          brne    sendDATA             ; jump if not
          cbi     PORTB,RS_TxD         ; send bit START
          inc     Tx_bit_count         ; count sent bits
          ret
          
sendDATA: cpi     Tx_bit_count,0x09    ; data bit?
          brsh    sendSTOP             ; jump if not
          sbrc    Tx_char,0            ; b0=1?
          sbi     PORTB,RS_TxD         ; send 1 (TxD=1)
          sbrs    Tx_char,0            ; b0=0?
          cbi     PORTB,RS_TxD         ; send 0 (TxD=0)
          lsr     Tx_char              ; prepare the next bit
          inc     Tx_bit_count         ; count sent bits
          ret
          
sendSTOP: sbi     PORTB,RS_TxD         ; send bit STOP
          clr     Tx_bit_count         ; clear sent bits counter
          cbr     flags,(1<<f_Tx_busy) ; deactivate RS transmitter (f_Tx_char=0)
          ret


; ---- RS receive character
; Reception of the RS232 bit is done in the middle of the bit defined by
; time_Rx register. Received character is completed in Rx_char buffer while
; set flag f_Rx_charOK signals the end of the RS character reception
;
get_char: cp      time_count,time_Rx
          breq    getSTART             ; the middle of the bit
          ret
          
getSTART: cpi     Rx_bit_count,0x00    ; START bit?
          brne    getDATA              ; jump if not
          sbic    PINB,RS_RxD          ; RxD=0?
          rjmp    getERROR             ; error - START bit is always 0
          inc     Rx_bit_count         ; count received bits
          ret
          
getDATA:  cpi     Rx_bit_count,0x09    ; data bit?
          brsh    getSTOP              ; jump if not
          clc                          ; C=0
          sbic    PINB,RS_RxD          ; RxD=1?
          sec                          ; C=1
          ror     Rx_char              ; save received bit
          inc     Rx_bit_count         ; count received bits
          ret
          
getSTOP:  sbis    PINB,RS_RxD          ; RxD=1?
          rjmp    getERROR             ; error - STOP bit is always 1
          sbr     flags,(1<<f_charOK)  ; character in Rx_char buffer (f_charOK)
          
getERROR: clr     Rx_bit_count         ; clear received bits counter
          cbr     flags,(1<<f_Rx_busy) ; deactivate RS receiver (f_Rx_char=0)
          ret


;-----------------------------------------------------------------------
;       External Interrupt 0 Service Subroutine (vector 0x01)
;-----------------------------------------------------------------------
; Read bits are stored in Wiegand buffer Wiegand_B0..Wiegand_B4 as:
;
;      Wiegand_B0:   b19 b20 b21 b22 b23 b24 b25 b26(odd)
;      Wiegand_B1:   b11 b12 b13 b14 b15 b16 b17 b18
;      Wiegand_B2:   b3  b4  b5  b6  b7  b8  b9  b10
;      Wiegand_B3:   x   x   x   x   x   x b1(even) b2
;      Wiegand_B4:   x   x   x   x   x   x   x   x
;        b1  - even parity bit for bits b2..b13
;        b26 - odd parity bit for bits b14..b25
; for Wiegand26 data format and as:
;
;      Wiegand_B0:   b30 b31 b32 b33 b34 b35 b36 b37(odd)
;      Wiegand_B1:   b22 b23 b24 b25 b26 b27 b28 b29
;      Wiegand_B2:   b14 b15 b16 b17 b18 b19 b20 b21
;      Wiegand_B3:   b6  b7  b8  b9  b10 b11 b12 b13
;      Wiegand_B4:   x   x   x b1(even) b2 b3 b4  b5
;        b1  - even parity bit for bits b2..b19
;        b37 - odd parity bit for bits b19..b37
; for Wiegand37 data format
; The end of the data frame is detected as time_Wgnd time-out
;
ext_INT0: in      SREGcopy,SREG        ; save SREG
          in      tempINT,PIND         ; read Wiagand inputs
          andi    tempINT,0b00000011   ; extract Data_1 and Data_0 lines
;         cpi     tempINT,0x00         ; Data_1Data_0=00?
          breq    extERROR             ; error - both lines can't be low
          cpi     tempINT,0x03         ; Data_1Data_0=11?
          breq    extERROR             ; error - one line should be low
          ror     tempINT              ; read Wiegand bit (C=0 when Data_0=0
                                       ;                   C=1 when Data_1=0)
          rol     Wiegand_B0           ; \ 
          rol     Wiegand_B1           ; |
          rol     Wiegand_B2           ; | complete Wiegand data block
          rol     Wiegand_B3           ; |
          rol     Wiegand_B4           ; /                  
          
          inc     Wgnd_count           ; count read bits
          ldi     time_Wgnd,Wgnd_tout  ; load time-out timer
          
extERROR: out     SREG,SREGcopy        ; restore SREG
          reti


;-----------------------------------------------------------------------
;                  Main Program (vector 0x00)
;-----------------------------------------------------------------------
reset:    
; ---- Clear RAM (initiate variables)
          clr     r31
          clr     r30                  ; initiate data pointer
clrRAM:   st      z,r31                ; clear RAM
          inc     r30                  ; next RAM location
          cpi     r30,30               ; all RAM cleared?
          brne    clrRAM


; ---- CPU Internal Hardware Configuration
; ---- Port B
;          PB7       - not used (in)
;          PB6       - not used (in)
;          PB5       - not used (in)
;          PB4       - not used (in)
;          PB3       - not used (in)
;          PB2       - not used (in)
;          PB1(AIN1) - RS232 - RxD (in)
;          PB0(AIN0) - RS232 - TxD (out)
; All inputs with pull-ups, outputs: TxD=1
          ldi     temp,0b11111111
          out     PORTB,temp           ; pull-ups on   
          ldi     temp,0b00000001
          out     DDRB,temp            ; PB7..PB1 - inputs, PB0 - output

; ---- Port D
;          PD6       - not used (in)
;          PD5       - Wiegand - Beeper (out)
;          PD4(T0)   - Wiegand - LED_Green (out)
;          PD3       - Wiegand - LED_Red (out)
;          PD2(INT0) - external interrupt (in)
;          PD1       - Wiegand - Data_1 (in)
;          PD0       - Wiegand - Data_0 (in)
; All inputs with pull-ups, outputs:  Beeper=0, LED_Green=0, LED_Red=0
          ldi     temp,0b01000111
          out     PORTD,temp           ; pull-ups on
          ldi     temp,0b00111000
          out     DDRD,temp            ; PD6,PD2..PD0 - inputs, PD5..PD3 - outputs

; ---- Timer T0
; Timer T0 generates clock interrupts every T0_prescaler*T0_period/Fosc=208.33us
; (i.e. four times faster than RS232 speed)
          ldi     temp,(256-T0_period)
          out     TCNT0,temp           ; load T0 start value
          ldi     temp,0b00000011      ; CS02,CS01,CS00=011 (T0_prescaler=64)
          out     TCCR0,temp           ; start T0

; ---- Watchdog Timer
          ldi     temp,0b00001110      ; WDE=1, WDP2,WDP1,WDP0=110
          out     WDTCR,temp           ; enable WDT, time-out 0.97s

; ---- Interrupts
          ldi     temp,0b00000010      ; TOIE0=1
          out     TIMSK,temp           ; enable T0 overflow interrupts
          
          ldi     temp,0b00000010      ; ISC01,ISC00=10
          out     MCUCR,temp           ; INT0 generated by the falling edge
          ldi     temp,0b01000000      ; INT0=1
          out     GIMSK,temp           ; enable INT0 interrupts
          sei                          ; enable interrupts


; ---- Main Loop
mainLoop: wdr                          ; watchdog reset

          sbrs    flags,f_charOK       ; character in Rx buffer?
          rjmp    anlTx

; ---- Analysis of the incoming frame          
; Due to lack of RAM to organize RS receive buffer, incoming characters
; are analyzed "on the fly" by the state machine
; Frame structure:
;                  0x02  S/R  r/g/b  0x03
;                   |     |     |     |____ End of frame character: ETX
;                   |     |     |__________ Device: "r"-red LED, "g"-green LED, "b"-beeper
;                   |     |________________ Command: "S"-set, "R"-reset
;                   |______________________ Start of frame character: STX
;
          mov     temp,Rx_char         ; load received character

; ---- RS Rx State 0 (start of frame)
anlRx_S0: cpi     temp,char_STX        ; start of frame?
          brne    anlRx_S1
          ldi     Rx_state,0x01        ; next state 1
          rjmp    anlRx_end

; ---- RS Rx State 1 (command)          
anlRx_S1: cpi     Rx_state,0x01        ; state 1?
          brne    anlRx_S2
anlRx_S1s:cpi     temp,'S'             ; set command?
          brne    anlRx_S1r
          ldi     Rx_state,0x02        ; next state 2 (set)
          rjmp    anlRx_end
anlRx_S1r:cpi     temp,'R'             ; reset command?
          brne    anlRx_err
          ldi     Rx_state,0x03        ; next state 3 (reset)
          rjmp    anlRx_end
          
; ---- RS Rx State 2 (set)          
anlRx_S2: cpi     Rx_state,0x02        ; state 2?
          brne    anlRx_S3
anlRx_S2r:cpi     temp,'r'             ; red LED?
          brne    anlRx_S2g
          sbi     PORTD,W_LED_R        ; red LED on
          rjmp    anlRx_err
anlRx_S2g:cpi     temp,'g'             ; green LED?
          brne    anlRx_S2b
          sbi     PORTD,W_LED_G        ; green LED on
          rjmp    anlRx_err
anlRx_S2b:cpi     temp,'b'             ; beeper?
          brne    anlRx_err
          sbi     PORTD,W_Beeper       ; beeper on
          rjmp    anlRx_err
          
; ---- RS Rx State 3 (reset)          
anlRx_S3: cpi     Rx_state,0x03        ; state 3?
          brne    anlRx_err
anlRx_S3r:cpi     temp,'r'             ; red LED?
          brne    anlRx_S3g
          cbi     PORTD,W_LED_R        ; red LED off
          rjmp    anlRx_err
anlRx_S3g:cpi     temp,'g'             ; green LED?
          brne    anlRx_S3b
          cbi     PORTD,W_LED_G        ; green LED off
          rjmp    anlRx_err
anlRx_S3b:cpi     temp,'b'             ; beeper?
          brne    anlRx_err
          cbi     PORTD,W_Beeper       ; beeper off

; ---- RS Rx Error
anlRx_err:ldi     Rx_state,0x00        ; return to state 0

anlRx_end:cbr     flags,(1<<f_charOK)  ; clear receiver buffer



anlTx:    sbrc    flags,f_Tx_busy      ; RS transmitter busy?
          rjmp    mainLoop             ; if yes, wait

; ---- RS Retransmition of Wiegand data block
; Due to lack of RAM to organize RS transmit buffer, transmition is based on
; the state machine
; Frame structure:
;                  0x02  S/L  .....  0x03
;                   |     |     |     |____ End of frame character: ETX
;                   |     |     |__________ Card number*): 6B for Wiegand 26, 9B for Wiegand 37
;                   |     |________________ Wiegand format: "S"-Wiegand 26, "L"-Wiegand 37
;                   |______________________ Start of frame character: STX
;
; *) Each byte of the card number is send as two ASCII characters

; ---- RS Tx State 0 (wait for Wiegand data block)
anlTx_S0: cpi     Tx_state,0x00        ; state 0?
          brne    anlTx_S1
          rjmp    anlTx_end
          
; ---- RS Tx State 1 (analysis of Wiegand data block)
anlTx_S1: cpi     Tx_state,0x01        ; state 1?
          brne    anlTx_S2
          ldi     temp,0b00000000      ; INT0=0
          out     GIMSK,temp           ; disable INT0 interrupts
; Wiegand parity test
          cpi     Wgnd_count,26        ; \ Wiegand
          breq    anlTx_S1p            ; /   26?
          cpi     Wgnd_count,37        ; \ Wiegand
          breq    anlTx_S1p            ; /   37?      
          rjmp    anlTx_err            ; other formats are not supported
anlTx_S1p:rcall   chkWgnd              ; check parity
          brtc    anlTx_S1n            ; parity error (T=1)?
          rjmp    anlTx_err
; Masking unused bits in Wiegand buffer
anlTx_S1n:ror     Wiegand_B4           ; \ 
          ror     Wiegand_B3           ; |
          ror     Wiegand_B2           ; | skip parity bit (odd)
          ror     Wiegand_B1           ; |
          ror     Wiegand_B0           ; /
          ldi     temp,0b00000111      ; \ mask parity
          and     Wiegand_B4,temp      ; /  bit (even)
          ldi     Tx_state,0x02        ; next state 2
          rjmp    anlTx_end
          
; ---- RS Tx State 2 (Start of frame character)
anlTx_S2: cpi     Tx_state,0x02        ; state 2?
          brne    anlTx_S3
          ldi     temp,char_STX        ; \ 
          mov     Tx_char,temp         ; | send STX character
          sbr     flags,(1<<f_Tx_busy) ; /
          ldi     Tx_state,0x03        ; next state 3
          rjmp    anlTx_end
          
; ---- RS Tx State 3 (Wiegand data format character)
anlTx_S3: cpi     Tx_state,0x03        ; state 3?
          brne    anlTx_S4
          ldi     temp,'S'             ; \ Wiegand
          ldi     tx_state,0x07        ; /    26
          cpi     Wgnd_count,26        ; \ Wiegand
          breq    anlTx_S3s            ; /    26?
          ldi     temp,'L'             ; \ Wiegand
          ldi     tx_state,0x04        ; /    37
anlTx_S3s:mov     Tx_char,temp         ; \ send Wiegand
          sbr     flags,(1<<f_Tx_busy) ; /   format
          rjmp    anlTx_end
          
; ---- RS Tx State 4 (Wiegand 37 data: b2b3b4)
anlTx_S4: cpi     Tx_state,0x04        ; state 4?
          brne    anlTx_S5
          mov     temp,Wiegand_B4
          rcall   snd_LSN              ; send LSN as ASCII
          ldi     Tx_state,0x05        ; next state 5
          rjmp    anlTx_end          
         
; ---- RS Tx State 5 (Wiegand 37 data: b5b6b7b8)
anlTx_S5: cpi     Tx_state,0x05        ; state 5?
          brne    anlTx_S6
          mov     temp,Wiegand_B3
          swap    temp
          rcall   snd_LSN              ; send LSN as ASCII
          ldi     Tx_state,0x06        ; next state 6
          rjmp    anlTx_end

; ---- RS Tx State 6 (Wiegand 37 data: b9b10b11b12)
anlTx_S6: cpi     Tx_state,0x06        ; state 6?
          brne    anlTx_S7
          mov     temp,Wiegand_B3
          rcall   snd_LSN              ; send LSN as ASCII
          ldi     Tx_state,0x07        ; next state 7
          rjmp    anlTx_end

; ---- RS Tx State 7 (Wiegand 37 data: b13b14b15b16, Wiegand 26 data: b2b3b4b5)
anlTx_S7: cpi     Tx_state,0x07        ; state 7?
          brne    anlTx_S8
          mov     temp,Wiegand_B2
          swap    temp
          rcall   snd_LSN              ; send LSN as ASCII
          ldi     Tx_state,0x08        ; next state 8
          rjmp    anlTx_end
          
; ---- RS Tx State 8 (Wiegand 37 data: b17b18b19b20, Wiegand 26 data: b6b7b8b9)
anlTx_S8: cpi     Tx_state,0x08        ; state 8?
          brne    anlTx_S9
          mov     temp,Wiegand_B2
          rcall   snd_LSN              ; send LSN as ASCII
          ldi     Tx_state,0x09        ; next state 9
          rjmp    anlTx_end

; ---- RS Tx State 9 (Wiegand 37 data: b21b22b23b24, Wiegand 26 data: b10b11b12b13)
anlTx_S9: cpi     Tx_state,0x09        ; state 9?
          brne    anlTx_S10
          mov     temp,Wiegand_B1
          swap    temp
          rcall   snd_LSN              ; send LSN as ASCII
          ldi     Tx_state,0x0A        ; next state 10
          rjmp    anlTx_end

; ---- RS Tx State 10 (Wiegand 37 data: b25b26b27b28, Wiegand 26 data: b14b15b16b17)
anlTx_S10:cpi     Tx_state,0x0A        ; state 10?
          brne    anlTx_S11
          mov     temp,Wiegand_B1
          rcall   snd_LSN              ; send LSN as ASCII
          ldi     Tx_state,0x0B        ; next state 11
          rjmp    anlTx_end
          
; ---- RS Tx State 11 (Wiegand 37 data: b29b30b31b32, Wiegand 26 data: b18b19b20b21)
anlTx_S11:cpi     Tx_state,0x0B        ; state 11?
          brne    anlTx_S12
          mov     temp,Wiegand_B0
          swap    temp
          rcall   snd_LSN              ; send LSN as ASCII
          ldi     Tx_state,0x0C        ; next state 12
          rjmp    anlTx_end

; ---- RS Tx State 12 (Wiegand 37 data: b33b34b35b36, Wiegand 26 data: b22b23b24b25)
anlTx_S12:cpi     Tx_state,0x0C        ; state 12?
          brne    anlTx_S13
          mov     temp,Wiegand_B0
          rcall   snd_LSN              ; send LSN as ASCII
          ldi     Tx_state,0x0D        ; next state 13
          rjmp    anlTx_end

; ---- RS Tx State 13 (End of frame character)
anlTx_S13:cpi     Tx_state,0x0D        ; state 13?
          brne    anlTx_err
          ldi     temp,char_ETX        ; \ 
          mov     Tx_char,temp         ; | send ETX character
          sbr     flags,(1<<f_Tx_busy) ; /          
                    
; ---- RS Tx Error
anlTx_err:clr     Wgnd_count           ; reset read bits counter
          ldi     temp,0b01000000      ; INT0=1
          out     GIMSK,temp           ; enable INT0 interrupts
          ldi     Tx_state,0x00        ; return to state 0

anlTx_end:rjmp    mainLoop


;-----------------------------------------------------------------------
;                   Main program subroutines
;-----------------------------------------------------------------------
; ---- Send LSN in temp register as ASCII character
snd_LSN:  andi    temp,0b00001111      ; mask MSN
          subi    temp,(0x100-0x30)    ; convert LSN to ASCII
          cpi     temp,0x3A            ; \ LSN = A..F?
          brlo    sndLSN_rs            ; /
          subi    temp,(0x100-0x07)    ; correction for LSN=A..F
sndLSN_rs:mov     Tx_char,temp         ; \   send
          sbr     flags,(1<<f_Tx_busy) ; / character
          ret


; ---- Check Wiegand block parity
; All bits in Wiegand block are tested and "1s" are counted. The lbs in "1s" counter
; indicates even (lsb=0) or odd (lsb=1) number of "1s". If parity error is detected,
; flag T is set. Otherwise flag T is cleared.
chkWgnd:  mov     reg_A,Wiegand_B0     ; \ 
          mov     reg_B,Wiegand_B1     ; |
          mov     reg_C,Wiegand_B2     ; | copy Wiegand buffer
          mov     reg_D,Wiegand_B3     ; |
          mov     reg_E,Wiegand_B4     ; /
          bst     Wiegand_B2,2         ; \ copy bit b19 (b19 in Wiegand 37
          bld     reg_E,5              ; /     block is checked twice!)
; check odd parity
          clr     temp                 ; initiate "1s" counter
          ldi     loop_count,13        ; number of bits tested in Wiegand 26 block
          cpi     Wgnd_count,26        ; \ Wiegand 26
          breq    chkW_odd             ; / data block?
          ldi     loop_count,19        ; number of bits tested in Wiegand 37 block
chkW_odd: ror     reg_E                ; \ 
          ror     reg_D                ; | 
          ror     reg_C                ; | shift bit to C
          ror     reg_B                ; |
          ror     reg_A                ; /
          brcc    chkW_odd1            ; bit=1?
          inc     temp                 ; count "1s"
chkW_odd1:dec     loop_count           ; count bits
          brne    chkW_odd             ; odd parity calculated?
          sbrs    temp,0               ; odd number of "1s" (lsb in temp=1)?
          rjmp    chkW_err
; check even parity
          clr     temp                 ; initiate "1s" counter
          ldi     loop_count,13        ; number of bits tested in Wiegand 26 block
          cpi     Wgnd_count,26        ; \ Wiegand 26
          breq    chkW_even            ; / data block?
          ldi     loop_count,19        ; number of bits tested in Wiegand 37 block          
chkW_even:ror     reg_E                ; \ 
          ror     reg_D                ; | 
          ror     reg_C                ; | shift bit to C
          ror     reg_B                ; |
          ror     reg_A                ; /
          brcc    chkW_evn1            ; bit=1?
          inc     temp                 ; count "1s"
chkW_evn1:dec     loop_count           ; count bits
          brne    chkW_even            ; even parity calculated?
          sbrc    temp,0               ; even number of "1s" (lsb in temp=0)?
          rjmp    chkW_err
          clt                          ; no parity error (T=0)
          ret          
; parity error
chkW_err: set                          ; parity error (T=1)
          ret          
